@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2006-2011 Inverse inc.
+  Copyright (C) 2006-2013 Inverse inc.
   Copyright (C) 2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -45,6 +45,7 @@
 
 @class SOGoAppointmentFolder;
 @class SOGoAppointmentFolders;
+@class SOGoContactFolder;
 @class SOGoDateFormatter;
 @class SOGoDomainDefaults;
 @class SOGoUserDefaults;
@@ -52,6 +53,8 @@
 @class SOGoUserProfile;
 @class SOGoUserSettings;
 
+@protocol SOGoSource;
+
 @interface SOGoUser : SoUser
 {
   SOGoUserDefaults *_defaults;
@@ -60,7 +63,7 @@
   SOGoUserFolder *homeFolder;
   NSString *currentPassword;
   NSString *loginInDomain;
-  NSString *language;
+  //NSString *language;
   NSArray *allEmails;
   NSMutableArray *mailAccounts;
   NSString *cn;
@@ -88,6 +91,7 @@
 
 /* properties */
 - (NSString *) domain;
+- (id <SOGoSource>) authenticationSource;
 
 - (NSArray *) allEmails;
 - (BOOL) hasEmail: (NSString *) email;
@@ -127,6 +131,7 @@
 - (SOGoUserFolder *) homeFolderInContext: (id) context;
 - (SOGoAppointmentFolders *) calendarsFolderInContext: (WOContext *) context;
 - (SOGoAppointmentFolder *) personalCalendarFolderInContext: (WOContext *) context;
+- (SOGoContactFolder *) personalContactsFolderInContext: (WOContext *) context;
 
 @end
 
