@@ -1,4 +1,3 @@
-var isSieveScriptsEnabled = false;
 var filters = [];
 var mailAccounts = null;
 var dialogs = {};
@@ -10,11 +9,29 @@ function savePreferences(sender) {
     if (sigList)
         sigList.disabled = false;
 
-    if ($("calendarCategoriesListWrapper")) {
+    if ($("appointmentsWhiteListWrapper"))
+        serializeAppointmentsWhiteList();
+
+    if ($("calendarCategoriesListWrapper"))
         serializeCalendarCategories();
-    }
-    if ($("contactsCategoriesListWrapper")) {
+
+    if ($("contactsCategoriesListWrapper"))
         serializeContactsCategories();
+
+    if ($("mailLabelsListWrapper"))
+        serializeMailLabels();
+
+    if (typeof mailCustomFromEnabled !== "undefined" && !emailRE.test($("email").value)) {
+        showAlertDialog(_("Please specify a valid sender address."));
+        sendForm = false;
+    }
+
+    if ($("replyTo")) {
+        var replyTo = $("replyTo").value;
+        if (!replyTo.blank() && !emailRE.test(replyTo)) {
+            showAlertDialog(_("Please specify a valid reply-to address."));
+            sendForm = false;
+        }
     }
 
     if ($("dayStartTime")) {
@@ -22,39 +39,77 @@ function savePreferences(sender) {
         var selectedStart = parseInt(start.options[start.selectedIndex].value);
         var end = $("dayEndTime");
         var selectedEnd = parseInt(end.options[end.selectedIndex].value);
+
         if (selectedStart >= selectedEnd) {
-            alert (_("Day start time must be prior to day end time."));
+            showAlertDialog (_("Day start time must be prior to day end time."));
             sendForm = false;
         }
     }
 
     if ($("enableVacation") && $("enableVacation").checked) {
-        if ($("autoReplyText").value.strip().length == 0
-            || $("autoReplyEmailAddresses").value.strip().length == 0) {
-            alert(_("Please specify your message and your email addresses for which you want to enable auto reply."));
+        if ($("autoReplyText").value.strip().length == 0 || $("autoReplyEmailAddresses").value.strip().length == 0) {
+            showAlertDialog(_("Please specify your message and your email addresses for which you want to enable auto reply."));
             sendForm = false;
         }
+
+        if ($("autoReplyText").value.strip().endsWith('\n.')) {
+            showAlertDialog(_("Your vacation message must not end with a single dot on a line."));
+            sendForm = false;
+        }
+
+        if ($("enableVacationEndDate") && $("enableVacationEndDate").checked) {
+            var e = $("vacationEndDate_date");
+            var endDate = e.inputAsDate();
+            var now = new Date();
+            if (isNaN(endDate.getTime()) || endDate.getTime() < now.getTime()) {
+                showAlertDialog(_("End date of your auto reply must be in the future."));
+                sendForm = false;
+            }
+        }
     }
 
     if ($("enableForward") && $("enableForward").checked) {
         var addresses = $("forwardAddress").value.split(",");
         for (var i = 0; i < addresses.length && sendForm; i++)
             if (!emailRE.test(addresses[i].strip())) {
-                alert(_("Please specify an address to which you want to forward your messages."));
+                showAlertDialog(_("Please specify an address to which you want to forward your messages."));
                 sendForm = false;
             }
     }
 
-    if (isSieveScriptsEnabled) {
+    if (typeof sieveCapabilities != "undefined") {
         var jsonFilters = prototypeIfyFilters();
         $("sieveFilters").setValue(Object.toJSON(jsonFilters));
     }
 
-    saveMailAccounts();
-
-    if (sendForm)
-        $("mainForm").submit();
+    if (sendForm) {
+        saveMailAccounts();
 
+        triggerAjaxRequest($("mainForm").readAttribute("action"), function (http) {
+            if (http.readyState == 4) {
+                var response = http.responseText.evalJSON(true);
+                if (http.status == 503) {
+                    showAlertDialog(_(response.textStatus));
+                }
+                else if (http.status == 200) {
+                    if (response.hasChanged == 1) {
+                        window.opener.location.reload();
+                        window.close();
+                    }
+                    else {
+                        window.close();
+                    }
+                }
+                else {
+                    showAlertDialog(_(response.textStatus));
+                }
+            }
+        },
+                           null,
+                           Form.serialize($("mainForm")), // excludes the file input
+                           { "Content-type": "application/x-www-form-urlencoded"}
+                          );
+    }
     return false;
 }
 
@@ -74,9 +129,11 @@ function prototypeIfyFilters() {
             }
         }
 
-        newFilter.actions = $([]);
-        for (var j = 0; j < filter.actions.length; j++) {
-            newFilter.actions.push($(filter.actions[j]));
+        if (filter.actions) {
+            newFilter.actions = $([]);
+            for (var j = 0; j < filter.actions.length; j++) {
+                newFilter.actions.push($(filter.actions[j]));
+            }
         }
         newFilters.push(newFilter);
     }
@@ -87,7 +144,7 @@ function prototypeIfyFilters() {
 function _setupEvents() {
     var widgets = [ "timezone", "shortDateFormat", "longDateFormat",
                     "timeFormat", "weekStartDay", "dayStartTime", "dayEndTime",
-                    "firstWeek", "messageCheck", "sortByThreads", 
+                    "firstWeek", "refreshViewCheck", "sortByThreads", "displayRemoteInlineImages",
                     "subscribedFoldersOnly", "language", "defaultCalendar",
                     "enableVacation" ];
     for (var i = 0; i < widgets.length; i++) {
@@ -97,14 +154,13 @@ function _setupEvents() {
         }
     }
 
-
     // We check for non-null elements as replyPlacementList and composeMessagesType
     // might not be present if ModulesConstraints disable those elements
     if ($("replyPlacementList"))
-    	$("replyPlacementList").observe ("change", onReplyPlacementListChange);
-    
+        $("replyPlacementList").on("change", onReplyPlacementListChange);
+
     if ($("composeMessagesType"))
-    	$("composeMessagesType").observe ("change", onComposeMessagesTypeChange);
+        $("composeMessagesType").on("change", onComposeMessagesTypeChange);
 
     // Note: we also monitor changes to the calendar categories.
     // See functions endEditable and onColorPickerChoice.
@@ -116,6 +172,12 @@ function _setupEvents() {
     }
 }
 
+function onBodyClickHandler(event) {
+    var target = getTarget(event);
+    if (!target.hasClassName('colorBox'))
+        $("colorPickerDialog").hide();
+}
+
 function onChoiceChanged(event) {
     var hasChanged = $("hasChanged");
     hasChanged.value = "1";
@@ -129,12 +191,12 @@ function addDefaultEmailAddresses(event) {
     else addresses = new Array();
 
     defaultAddresses.each(function(adr) {
-            for (var i = 0; i < addresses.length; i++)
-                if (adr == addresses[i])
-                    break;
-            if (i == addresses.length)
-                addresses.push(adr);
-        });
+        for (var i = 0; i < addresses.length; i++)
+            if (adr == addresses[i])
+                break;
+        if (i == addresses.length)
+            addresses.push(adr);
+    });
 
     $("autoReplyEmailAddresses").value = addresses.join(", ");
 
@@ -146,14 +208,83 @@ function initPreferences() {
     var controller = new SOGoTabsController();
     controller.attachToTabsContainer(tabsContainer);
 
-    var filtersListWrapper = $("filtersListWrapper");
-    if (filtersListWrapper) {
-        isSieveScriptsEnabled = true;
+    // Inner tabs on the mail module tab
+    tabsContainer = $('mailOptionsTabs');
+    if (tabsContainer) {
+        var mailController = new SOGoTabsController();
+        mailController.attachToTabsContainer(tabsContainer);
     }
+
+    // Inner tabs on the calendar module tab
+    tabsContainer = $('calendarOptionsTabs');
+    if (tabsContainer) {
+        var mailController = new SOGoTabsController();
+        mailController.attachToTabsContainer(tabsContainer);
+    }
+
     _setupEvents();
+
+    // Optional function called when initializing the preferences
+    // Typically defined inline in the UIxAdditionalPreferences.wox template
     if (typeof (initAdditionalPreferences) != "undefined")
         initAdditionalPreferences();
 
+    // Color picker
+    $('colorPickerDialog').on('click', 'span', onColorPickerChoice);
+    $(document.body).on("click", onBodyClickHandler);
+
+    // Calendar whiteList
+    var whiteList = $("appointmentsWhiteListWrapper");
+    if (whiteList) {
+        var whiteListString = $("whiteList").getValue();
+        var whiteListObject = {};
+        // This condition is a backward compatibility where the strings looks like : "sogo1=John DOE <sogo1@example.com>"
+        if (whiteListString.search("=") != -1) {
+            var split = whiteListString.split("=");
+            whiteListObject[split[0]] = split[1];
+            
+        }
+        else if (whiteListString != "") {
+            whiteListObject = JSON.parse(whiteListString);
+        }
+        var allKeys = Object.keys(whiteListObject);
+        var allValues = Object.values(whiteListObject);
+        var tablebody = $("appointmentsWhiteListWrapper").childNodesWithTag("table")[0].tBodies[0];
+        for (i = 0; i < allKeys.length; i++) {
+            var row = new Element("tr");
+            var td = new Element("td").update("");
+            var textField = new Element("input");
+            var span = new Element("span");
+            
+            row.addClassName("whiteListRow");
+            row.observe("mousedown", onRowClick);
+            td.addClassName ("whiteListCell");
+            td.observe("mousedown", endAllEditables);
+            td.observe("dblclick", onNameEdit);
+            textField.addInterface(SOGoAutoCompletionInterface);
+            textField.SOGoUsersSearch = true;
+            textField.observe("autocompletion:changed", endEditable);
+            textField.addClassName("textField");
+            textField.value = allValues[i];
+            textField.setAttribute("uid", allKeys[i]);
+            textField.hide();
+            span.innerText = allValues[i];
+            
+            td.appendChild(textField);
+            td.appendChild(span);
+            row.appendChild (td);
+            tablebody.appendChild(row);
+            $(tablebody).deselectAll();
+            
+        }
+        
+        var table = whiteList.childNodesWithTag("table")[0];
+        table.multiselect = true;
+        $("appointmentsWhiteListAdd").observe("click", onAppointmentsWhiteListAdd);
+        $("appointmentsWhiteListDelete").observe("click", onAppointmentsWhiteListDelete);
+    }
+
+    // Calender categories
     var wrapper = $("calendarCategoriesListWrapper");
     if (wrapper) {
         var table = wrapper.childNodesWithTag("table")[0];
@@ -165,8 +296,24 @@ function initPreferences() {
         resetCalendarTableActions();
         $("calendarCategoryAdd").observe("click", onCalendarCategoryAdd);
         $("calendarCategoryDelete").observe("click", onCalendarCategoryDelete);
+        wrapper.observe("scroll", onBodyClickHandler);
     }
 
+    // Mail labels/tags
+    var wrapper = $("mailLabelsListWrapper");
+    if (wrapper) {
+        var table = wrapper.childNodesWithTag("table")[0];
+        resetMailLabelsColors(null);
+        var r = $$("#mailLabelsListWrapper tbody tr");
+        for (var i= 0; i < r.length; i++)
+            r[i].identify();
+        table.multiselect = true;
+        resetMailTableActions();
+        $("mailLabelAdd").observe("click", onMailLabelAdd);
+        $("mailLabelDelete").observe("click", onMailLabelDelete);
+    }
+
+    // Contact categories
     wrapper = $("contactsCategoriesListWrapper");
     if (wrapper) {
         var table = wrapper.childNodesWithTag("table")[0];
@@ -179,14 +326,8 @@ function initPreferences() {
         $("contactsCategoryDelete").observe("click", onContactsCategoryDelete);
     }
 
-    // Disable placement (after) if composing in HTML
-    if ($("composeMessagesType")) {
-        if ($("composeMessagesType").value == 1) {
-            $("replyPlacementList").selectedIndex = 0;
-            $("replyPlacementList").disabled = 1;
-        }
-        onReplyPlacementListChange ();
-    }
+    if ($("replyPlacementList"))
+        onReplyPlacementListChange();
 
     var button = $("addDefaultEmailAddresses");
     if (button)
@@ -197,7 +338,20 @@ function initPreferences() {
         button.observe("click", onChangePasswordClick);
 
     initSieveFilters();
+
     initMailAccounts();
+
+    button = $("enableVacationEndDate");
+    if (button) {
+        jQuery("#vacationEndDate_date").closest(".date").datepicker({ autoclose: true, position: 'above', weekStart: $('weekStartDay').getValue() });
+        button.on("click", function(event) {
+            if (this.checked)
+                $("vacationEndDate_date").enable();
+            else
+                $("vacationEndDate_date").disable();
+        });
+    }
+    onAddOutgoingAddressesCheck();
 }
 
 function initSieveFilters() {
@@ -258,7 +412,7 @@ function updateSieveFilterRow(filterTable, number, filter) {
 }
 
 function _editFilter(filterId) {
-    var urlstr = ApplicationBaseURL + "editFilter?filter=" + filterId;
+    var urlstr = ApplicationBaseURL + "/editFilter?filter=" + filterId;
     var win = window.open(urlstr, "sieve_filter_" + filterId,
                           "width=560,height=380,resizable=0");
     if (win)
@@ -279,7 +433,7 @@ function onFilterDelete(event) {
             var node = nodes[i];
             deletedFilters.push(node.rowIndex - 1);
         }
-        deletedFilters = deletedFilters.sort(function (x,y) { return x-y; });
+        deletedFilters = deletedFilters.sort(function(x,y) { return x-y; });
         var rows = filtersList.rows;
         for (var i = 0; i < deletedFilters.length; i++) {
             var filterNbr = deletedFilters[i];
@@ -330,11 +484,12 @@ function onFilterMoveDown(event) {
 function onFilterEdit(event) {
     _editFilter(this.rowIndex - 1);
     event.stop();
+
+    return false;
 }
 
 function copyFilter(originalFilter) {
     var newFilter = {};
-
     newFilter.name = originalFilter.name;
     newFilter.match = originalFilter.match;
     newFilter.active = originalFilter.active;
@@ -369,15 +524,17 @@ function getSieveCapabilitiesFromEditor() {
 }
 
 function getFilterFromEditor(filterId) {
-    return copyFilter(filters[filterId]);
+    var filter = copyFilter(filters[filterId]);
+    return Object.toJSON(filter);
 }
 
 function setupMailboxesFromJSON(jsonResponse) {
     var responseMboxes = jsonResponse.mailboxes;
     userMailboxes = $([]);
     for (var i = 0; i < responseMboxes.length; i++) {
-        var name = responseMboxes[i].path.substr(1);
-        userMailboxes.push(name);
+        var mbox = { 'displayName': responseMboxes[i].displayName.substr(1),
+                     'path': responseMboxes[i].path.substr(1) };
+        userMailboxes.push(mbox);
     }
 }
 
@@ -404,52 +561,51 @@ function updateFilterFromEditor(filterId, filterJSON) {
 /* mail accounts */
 function initMailAccounts() {
     var mailAccountsJSON = $("mailAccountsJSON");
-    mailAccounts = mailAccountsJSON.value.evalJSON();
+    if (mailAccountsJSON) {
+        mailAccounts = mailAccountsJSON.value.evalJSON();
 
-    var mailAccountsList = $("mailAccountsList");
-    if (mailAccountsList) {
-        var li = createMailAccountLI(mailAccounts[0], true);
-        mailAccountsList.appendChild(li);
-        for (var i = 1; i < mailAccounts.length; i++) {
-            li = createMailAccountLI(mailAccounts[i]);
+        var mailAccountsList = $("mailAccountsList");
+        if (mailAccountsList) {
+            var li = createMailAccountLI(mailAccounts[0], true);
             mailAccountsList.appendChild(li);
+            for (var i = 1; i < mailAccounts.length; i++) {
+                li = createMailAccountLI(mailAccounts[i]);
+                mailAccountsList.appendChild(li);
+            }
+            var lis = mailAccountsList.childNodesWithTag("li");
+            lis[0].readOnly = true;
+            lis[0].selectElement();
+
+            var button = $("mailAccountAdd");
+            if (button) {
+                button.observe("click", onMailAccountAdd);
+            }
+            button = $("mailAccountDelete");
+            if (button) {
+                button.observe("click", onMailAccountDelete);
+            }
         }
-        var lis = mailAccountsList.childNodesWithTag("li");
-        lis[0].readOnly = true;
-        lis[0].selectElement();
 
-        var button = $("mailAccountAdd");
-        if (button) {
-            button.observe("click", onMailAccountAdd);
+        var inputs = $$("#accountInfo input");
+        for (var i = 0; i < inputs.length; i++) {
+            $(inputs[i]).observe("change", onMailAccountInfoChange);
         }
-        button = $("mailAccountDelete");
-        if (button) {
-            button.observe("click", onMailAccountDelete);
+
+        inputs = $$("#identityInfo input");
+        for (var i = 0; i < inputs.length; i++) {
+            $(inputs[i]).observe("change", onMailIdentityInfoChange);
         }
-    }
+        $("actSignature").observe("click", onMailIdentitySignatureClick);
+        displayMailAccount(mailAccounts[0], true);
 
-    var info = $("accountInfo");
-    var inputs = info.getElementsByTagName("input");
-    for (var i = 0; i < inputs.length; i++) {
-        $(inputs[i]).observe("change", onMailAccountInfoChange);
-    }
-
-    info = $("identityInfo");
-    inputs = info.getElementsByTagName("input");
-    for (var i = 0; i < inputs.length; i++) {
-        $(inputs[i]).observe("change", onMailIdentityInfoChange);
-    }
-    $("actSignature").observe("click", onMailIdentitySignatureClick);
-    displayMailAccount(mailAccounts[0], true);
-
-    info = $("returnReceiptsInfo");
-    inputs = info.getElementsByTagName("input");
-    for (var i = 0; i < inputs.length; i++) {
-        $(inputs[i]).observe("change", onMailReceiptInfoChange);
-    }
-    inputs = info.getElementsByTagName("select");
-    for (var i = 0; i < inputs.length; i++) {
-        $(inputs[i]).observe("change", onMailReceiptActionChange);
+        inputs = $$("#returnReceiptsInfo input");
+        for (var i = 0; i < inputs.length; i++) {
+            $(inputs[i]).observe("change", onMailReceiptInfoChange);
+        }
+        inputs = $$("#returnReceiptsInfo select");
+        for (var i = 0; i < inputs.length; i++) {
+            $(inputs[i]).observe("change", onMailReceiptActionChange);
+        }
     }
 }
 
@@ -502,26 +658,28 @@ function onMailIdentitySignatureClick(event) {
             var label = _("Please enter your signature below:");
             var fields = createElement("p");
             fields.appendChild(createElement("textarea", "signature"));
-            fields.appendChild(createElement("br"));
             fields.appendChild(createButton("okBtn", _("OK"),
                                             onMailIdentitySignatureOK));
             fields.appendChild(createButton("cancelBtn", _("Cancel"),
-                                            disposeDialog.bind(document.body, dialogId)));
+                                            disposeDialog));
             var dialog = createDialog(dialogId,
                                       _("Signature"),
                                       label,
                                       fields,
                                       "none");
+            if (Prototype.Browser.IE)
+                // Overwrite some fixes from iefixes.css
+                dialog.setStyle({ width: 'auto', marginLeft: 'auto' });
+
             document.body.appendChild(dialog);
-            dialog.show();
             dialogs[dialogId] = dialog;
 
             if ($("composeMessagesType").value != 0) {
                 CKEDITOR.replace('signature',
-                                 { height: "70px",
+                                 { height: "150px",
                                    toolbar: [['Bold', 'Italic', '-', 'Link',
                                               'Font','FontSize','-','TextColor',
-                                              'BGColor']
+                                              'BGColor'], ['Source']
                                             ],
                                    language: localeCode,
                                    scayt_sLang: localeCode });
@@ -537,14 +695,34 @@ function onMailIdentitySignatureClick(event) {
             area.value = identity["signature"];
         else
             area.value = "";
-        dialog.show();
+
         $("bgDialogDiv").show();
-        if (!CKEDITOR.instances["signature"])
-            area.focus();
-        event.stop();
+        if (Prototype.Browser.IE)
+            jQuery('#bgDialogDiv').css('opacity', 0.4);
+        jQuery(dialog).fadeIn('fast', function() {
+            if (CKEDITOR.instances["signature"])
+                focusCKEditor();
+            else
+                area.focus();
+        });
+        Event.stop(event);
     }
 }
 
+function focusCKEditor() {
+    if (CKEDITOR.status != 'loaded')
+        setTimeout("focusCKEditor()", 100);
+    else
+        CKEDITOR.instances.signature.focus()
+}
+
+function hideSignature() {
+    if (CKEDITOR.status != 'loaded')
+        setTimeout("hideSignature()", 100);
+    else
+        disposeDialog("signatureDialog");
+}
+
 function onMailIdentitySignatureOK(event) {
     var dialog = $("signatureDialog");
     var mailAccount = dialog.mailAccount;
@@ -558,8 +736,7 @@ function onMailIdentitySignatureOK(event) {
                    : $("signature").value);
     identity["signature"] = content;
     displayAccountSignature(mailAccount);
-    dialog.hide();
-    $("bgDialogDiv").hide();
+    hideSignature();
     dialog.mailAccount = null;
     var hasChanged = $("hasChanged");
     hasChanged.value = "1";
@@ -591,17 +768,18 @@ function onMailAccountEntryClick(event) {
 }
 
 function displayMailAccount(mailAccount, readOnly) {
-    var fieldSet = $("accountInfo");
-    var inputs = fieldSet.getElementsByTagName("input");
-    for (var i = 0; i < inputs.length; i++) {
-        inputs[i].disabled = readOnly;
-        inputs[i].mailAccount = mailAccount;
-    }
-    fieldSet = $("identityInfo");
-    inputs = fieldSet.getElementsByTagName("input");
-    for (var i = 0; i < inputs.length; i++) {
-        inputs[i].disabled = readOnly;
-        inputs[i].mailAccount = mailAccount;
+    var inputs = $$("#accountInfo input");
+    inputs.each(function(i) {
+        i.disabled = readOnly;
+        i.mailAccount = mailAccount;
+    });
+
+    inputs = $$("#identityInfo input");
+    inputs.each(function(i) { i.mailAccount = mailAccount; });
+    if (!mailCustomFromEnabled) {
+        for (var i = 0; i < 2; i++) {
+            inputs[i].disabled = readOnly;
+        }
     }
 
     var form = $("mainForm");
@@ -636,6 +814,7 @@ function displayMailAccount(mailAccount, readOnly) {
                     : {} );
     $("fullName").value = identity["fullName"] || "";
     $("email").value = identity["email"] || "";
+    $("replyTo").value = identity["replyTo"] || "";
 
     displayAccountSignature(mailAccount);
 
@@ -760,16 +939,17 @@ function onMailAccountDelete(event) {
 
 function saveMailAccounts() {
     /* This removal enables us to avoid a few warning from SOPE for the inputs
-     that were created dynamically. */
+       that were created dynamically. */
     var editor = $("mailAccountEditor");
-    
+
     // Could be null if ModuleConstraints disables email access
     if (editor)
-    	editor.parentNode.removeChild(editor);
+        //Instead of removing the modules, we disable it. This will prevent the window to crash if we have a connection error.
+        editor.select('input, select').each(function(i) { i.disable(); })
 
-    compactMailAccounts();
+            compactMailAccounts();
     var mailAccountsJSON = $("mailAccountsJSON");
- 
+
     if (mailAccountsJSON)
         mailAccountsJSON.value = Object.toJSON(mailAccounts);
 }
@@ -800,6 +980,45 @@ function compactMailAccounts() {
     }
 }
 
+/* common function between calendar categories and mail labels */
+function onColorEdit(e, target) {
+    var view = $(target);
+
+    view.select('div.colorEditing').each(function(box) {
+        box.removeClassName('colorEditing');
+    });
+    this.addClassName("colorEditing");
+
+    var cellPosition = this.cumulativeOffset();
+    var cellDimensions = this.getDimensions();
+    var div = $('colorPickerDialog');
+    var divDimensions = div.getDimensions();
+    var left = cellPosition[0] - divDimensions["width"];
+    var top = cellPosition[1] - 165 - view.scrollTop;
+    div.setStyle({ left: left + "px", top: top + "px" });
+    div.writeAttribute('data-target', target);
+    div.show();
+}
+
+function onColorPickerChoice(event) {
+    var span = getTarget(event);
+    var dialog = span.up('.dialog');
+    var target = dialog.readAttribute('data-target');
+    var newColor = "#" + span.className.substr(4);
+
+    var wrapper = $(target);
+    var div = wrapper.select("div.colorEditing").first();
+
+    div.writeAttribute('data-color', newColor);
+    div.style.background = newColor;
+    if (parseInt($("hasChanged").value) == 0) {
+        var hasChanged = $("hasChanged");
+        hasChanged.value = "1";
+    }
+
+    dialog.hide();
+}
+
 /* calendar categories */
 function resetCalendarTableActions() {
     var r = $$("#calendarCategoriesListWrapper tbody tr");
@@ -809,70 +1028,157 @@ function resetCalendarTableActions() {
         var tds = row.childElements();
         var editionCtlr = new RowEditionController();
         editionCtlr.attachToRowElement(tds[0]);
-        tds[1].childElements()[0].observe("dblclick", onColorEdit);
+        tds[1].childElements()[0].observe("click", onCalendarColorEdit);
     }
 }
 
-function onColorEdit (e) {
-    var r = $$("#calendarCategoriesListWrapper div.colorEditing");
-    for (var i=0; i<r.length; i++)
-        r[i].removeClassName("colorEditing");
-
-    this.addClassName ("colorEditing");
-    var cPicker = window.open(ApplicationBaseURL + "../" + UserLogin
-                              + "/Calendar/colorPicker", "colorPicker",
-                              "width=250,height=200,resizable=0,scrollbars=0"
-                              + "toolbar=0,location=0,directories=0,status=0,"
-                              + "menubar=0,copyhistory=0", "test"
-                              );
-    cPicker.focus();
-
-    preventDefault(e);
+function onCalendarColorEdit(e) {
+    var onCCE = onColorEdit.bind(this);
+    onCCE(e, "calendarCategoriesListWrapper");
 }
 
-function onColorPickerChoice (newColor) {
-    var div = $$("#calendarCategoriesListWrapper div.colorEditing").first ();
-    //  div.removeClassName ("colorEditing");
-    div.showColor = newColor;
-    div.style.background = newColor;
-    if (parseInt($("hasChanged").value) == 0) {
-        var hasChanged = $("hasChanged");
-        hasChanged.value = "1";
+function makeEditable (element) {
+    element.addClassName("editing");
+    element.removeClassName("whiteListCell");
+
+    var span = element.down("SPAN");
+    span.update();
+
+    var textField = element.down("INPUT");
+    textField.show();
+    textField.focus();
+    textField.select();
+
+    return true;
+}
+
+function endAllEditables (e) {
+    var r = $$("TABLE#tableViewWhiteList TBODY TR TD");
+    for (var i = 0; i < r.length; i++) {
+        var element = $(r[i]);
+        if (r[i] != this && element.hasClassName("editing"))
+            endEditable(null, element.down("INPUT"));
     }
 }
 
-function onCalendarCategoryAdd (e) {
-    var row = new Element ("tr");
-    var nametd = new Element ("td").update ("");
-    var colortd = new Element ("td");
-    var colordiv = new Element ("div", {"class": "colorBox"});
+function onNameEdit (e) {
+    endAllEditables();
+    if (!this.hasClassName("editing")) {
+        makeEditable (this);
+    }
+}
 
-    row.identify ();
-    row.addClassName ("categoryListRow");
+function endEditable(event, textField) {
+    if (!textField)
+        textField = this;
 
-    nametd.addClassName ("categoryListCell");
-    colortd.addClassName ("categoryListCell");
-    colordiv.innerHTML = "&nbsp;";
-    colordiv.showColor = "#F0F0F0";
-    colordiv.style.background = colordiv.showColor;
+    var uid = textField.readAttribute("uid");
+    var cell = textField.up("TD");
+    var textSpan = cell.down("SPAN");
 
-    colortd.appendChild (colordiv);
-    row.appendChild (nametd);
-    row.appendChild (colortd);
-    $("calendarCategoriesListWrapper").childNodesWithTag("table")[0].tBodies[0].appendChild (row);
+    cell.removeClassName("editing");
+    cell.addClassName("whiteListCell");
+    textField.hide();
 
-    resetCalendarTableActions ();
+    var tmp = textField.value;
+    tmp = tmp.replace (/</, "&lt;");
+    tmp = tmp.replace (/>/, "&gt;");
+    if (!uid)
+        cell.up("TR").addClassName("notfound");
+    if (tmp)
+        textSpan.update(tmp);
+    else
+        cell.up("TR").remove();
+
+    if (event)
+        Event.stop(event);
+
+    return false;
+}
+
+function onAppointmentsWhiteListAdd(e) {
+    var tablebody = $("appointmentsWhiteListWrapper").childNodesWithTag("table")[0].tBodies[0];
+    var row = new Element("tr");
+    var td = new Element("td").update("");
+    var textField = new Element("input");
+    var span = new Element("span");
+
+    row.addClassName("whiteListRow");
+    row.observe("mousedown", onRowClick);
+    td.addClassName ("whiteListCell");
+    td.observe("mousedown", endAllEditables);
+    td.observe("dblclick", onNameEdit);
+    textField.addInterface(SOGoAutoCompletionInterface);
+    textField.SOGoUsersSearch = true;
+    textField.observe("autocompletion:changed", endEditable);
+    textField.addClassName("textField");
+
+    td.appendChild(textField);
+    td.appendChild(span);
+    row.appendChild (td);
+    tablebody.appendChild(row);
+    $(tablebody).deselectAll();
+    row.selectElement();
+
+    makeEditable(td);
+
+}
+
+function onAppointmentsWhiteListDelete(e) {
+    var list = $('appointmentsWhiteListWrapper').down("TABLE").down("TBODY");
+    var rows = list.getSelectedNodes();
+    var count = rows.length;
+
+    for (var i=0; i < count; i++) {
+        rows[i].editionController = null;
+        rows[i].remove();
+    }
+}
+
+function serializeAppointmentsWhiteList() {
+    var r = $$("#appointmentsWhiteListWrapper TBODY TR");
+
+    var users = {};
+    for (var i = 0; i < r.length; i++) {
+        var tds = r[i].childElements().first().down("INPUT");
+        var uid  = tds.getAttribute("uid");
+        var value = tds.getValue();
+        if (uid != null)
+            users[uid] = value;
+    }
+    $("whiteList").value = Object.toJSON(users);
+}
+
+function onCalendarCategoryAdd(e) {
+    var row = new Element("tr");
+    var nametd = new Element("td").update("");
+    var colortd = new Element("td");
+    var colordiv = new Element("div", {"class": "colorBox", dataColor: "#F0F0F0"});
+
+    row.identify();
+    row.addClassName("categoryListRow");
+
+    nametd.addClassName("categoryListCell");
+    colortd.addClassName("categoryListCell");
+    colordiv.setStyle({backgroundColor: "#F0F0F0"});
+
+    colortd.appendChild(colordiv);
+    row.appendChild(nametd);
+    row.appendChild(colortd);
+    $("calendarCategoriesListWrapper").childNodesWithTag("table")[0].tBodies[0].appendChild(row);
+
+    resetCalendarTableActions();
     nametd.editionController.startEditing();
 }
 
-function onCalendarCategoryDelete (e) {
+function onCalendarCategoryDelete(e) {
     var list = $('calendarCategoriesListWrapper').down("TABLE").down("TBODY");
     var rows = list.getSelectedNodes();
     var count = rows.length;
 
     for (var i=0; i < count; i++) {
         rows[i].editionController = null;
-        rows[i].remove ();
+        rows[i].remove();
     }
 }
 
@@ -881,29 +1187,112 @@ function serializeCalendarCategories() {
 
     var values = [];
     for (var i = 0; i < r.length; i++) {
-        var tds = r[i].childElements ();
-        var name  = $(tds.first ()).innerHTML;
-        var color = $(tds.last ().childElements ().first ()).showColor;
+        var tds = r[i].childElements();
+        var name  = $(tds.first()).innerHTML.trim();
+        var color = $(tds.last().childElements().first()).readAttribute('data-color');
         values.push("\"" + name + "\": \"" + color + "\"");
     }
 
     $("calendarCategoriesValue").value = "{ " + values.join(",\n") + "}";
 }
 
-function resetCalendarCategoriesColors (e) {
+function resetCalendarCategoriesColors(e) {
     var divs = $$("#calendarCategoriesListWrapper DIV.colorBox");
     for (var i = 0; i < divs.length; i++) {
         var d = divs[i];
-        var color = d.innerHTML;
-        d.showColor = color;
+        var color = d.readAttribute("data-color");
         if (color != "undefined")
             d.setStyle({ backgroundColor: color });
-        d.update("&nbsp;");
     }
 }
 
 /* /calendar categories */
 
+/* mail label/tags */
+function resetMailTableActions() {
+    var r = $$("#mailLabelsListWrapper tbody tr");
+    for (var i = 0; i < r.length; i++) {
+        var row = $(r[i]);
+        row.observe("mousedown", onRowClick);
+        var tds = row.childElements();
+        var editionCtlr = new RowEditionController();
+        editionCtlr.attachToRowElement(tds[0]);
+        tds[1].childElements()[0].observe("click", onMailColorEdit);
+    }
+}
+
+function onMailColorEdit(e) {
+    var onMCE = onColorEdit.bind(this);
+    onMCE(e, "mailLabelsListWrapper");
+}
+
+function onMailLabelAdd(e) {
+    var row = new Element("tr");
+    var nametd = new Element("td").update("");
+    var colortd = new Element("td");
+    var colordiv = new Element("div", {"class": "colorBox", dataColor: "#F0F0F0"});
+
+    row.identify();
+    row.addClassName("labelListRow");
+
+    nametd.addClassName("labelListCell");
+    colortd.addClassName("labelListCell");
+    colordiv.setStyle({backgroundColor: "#F0F0F0"});
+
+    colortd.appendChild(colordiv);
+    row.appendChild(nametd);
+    row.appendChild(colortd);
+    $("mailLabelsListWrapper").childNodesWithTag("table")[0].tBodies[0].appendChild(row);
+
+    resetMailTableActions();
+    nametd.editionController.startEditing();
+}
+
+function onMailLabelDelete(e) {
+    var list = $('mailLabelsListWrapper').down("TABLE").down("TBODY");
+    var rows = list.getSelectedNodes();
+    var count = rows.length;
+
+    for (var i=0; i < count; i++) {
+        rows[i].editionController = null;
+        rows[i].remove();
+    }
+}
+
+function resetMailLabelsColors(e) {
+    var divs = $$("#mailLabelsListWrapper DIV.colorBox");
+    for (var i = 0; i < divs.length; i++) {
+        var d = divs[i];
+        var color = d.readAttribute('data-color');
+        if (color != "undefined")
+            d.setStyle({ backgroundColor: color });
+    }
+}
+
+function serializeMailLabels() {
+    var r = $$("#mailLabelsListWrapper TBODY TR");
+
+    var values = [];
+    for (var i = 0; i < r.length; i++) {
+        var tds = r[i].childElements();
+        var name = r[i].readAttribute("data-name"); 
+        var label  = $(tds.first()).innerHTML;
+        var color = $(tds.last().childElements().first()).readAttribute('data-color');
+
+        /* if name is null, that's because we've just added a new tag */
+        if (!name) {
+            name = label.replace(/[ \(\)\/\{%\*<>\\\"]/g, "_");
+        }
+
+        values.push("\"" + name + "\": [\"" + label + "\", \"" + color + "\"]");
+    }
+
+    $("mailLabelsValue").value = "{ " + values.join(",\n") + "}";
+}
+
+/* /mail label/tags */
+
+
 /* contacts categories */
 function resetContactsTableActions() {
     var r = $$("#contactsCategoriesListWrapper tbody tr");
@@ -927,11 +1316,11 @@ function onContactsCategoryAdd(e) {
     var list = $('contactsCategoriesListWrapper').down("TABLE").down("TBODY");
     list.appendChild(row);
 
-    resetContactsTableActions ();
+    resetContactsTableActions();
     nametd.editionController.startEditing();
 }
 
-function onContactsCategoryDelete (e) {
+function onContactsCategoryDelete(e) {
     var list = $('contactsCategoriesListWrapper').down("TABLE").down("TBODY");
     var rows = list.getSelectedNodes();
     var count = rows.length;
@@ -957,15 +1346,22 @@ function serializeContactsCategories() {
 
 /* / contact categories */
 
+function onAddOutgoingAddressesCheck(checkBox) {
+    if (!checkBox) {
+        checkBox = $("addOutgoingAddresses");
+    }
+    $("addressBookList").disabled = !checkBox.checked;
+}
 
 function onReplyPlacementListChange() {
-    // above = 0
     if ($("replyPlacementList").value == 0) {
-        $("signaturePlacementList").disabled=false;
+        // Reply placement is above quote, signature can be place before of after quote
+        $("signaturePlacementList").disabled = false;
     }
     else {
-        $("signaturePlacementList").value=1;
-        $("signaturePlacementList").disabled=true;
+        // Reply placement is bellow quote, signature is unconditionally placed after quote
+        $("signaturePlacementList").value = 1;
+        $("signaturePlacementList").disabled = true;
     }
 }
 
@@ -1015,13 +1411,10 @@ function onChangePasswordClick(event) {
                 policy.changePassword(password);
             }
             else
-                SetLogMessage("passwordError", _("Password must not be empty."),
-                                    "error");
+                SetLogMessage("passwordError", _("Password must not be empty."), "error");
         }
         else {
-            SetLogMessage("passwordError", _("The passwords do not match."
-                                  + " Please try again."),
-                                "error");
+            SetLogMessage("passwordError", _("The passwords do not match. Please try again."), "error");
             field.focus();
             field.select();
         }
